@@ -4,6 +4,8 @@
 #include <stdio.h>
 #include "corba-ops.h"
 #include "orb-core-private.h"
+#include "orb-core-export.h"
+#include "../util/orbit-purify.h"
 
 #undef DEBUG
 
@@ -16,11 +18,11 @@ static GHashTable *objrefs = NULL;
 static guint
 g_CORBA_Object_hash (gconstpointer key)
 {
-	guint retval;
-	CORBA_Object _obj = (gpointer) key;
+	guint        retval;
+	CORBA_Object obj = (gpointer) key;
 
-	retval = g_str_hash (_obj->type_id);
-	g_slist_foreach (_obj->profile_list, IOP_profile_hash, &retval);
+	retval = obj->type_qid;
+	g_slist_foreach (obj->profile_list, IOP_profile_hash, &retval);
 
 	return retval;
 }
@@ -32,11 +34,6 @@ g_CORBA_Object_equal (gconstpointer a, gconstpointer b)
 	CORBA_Object _obj = (CORBA_Object) a;
 	CORBA_Object other_object = (CORBA_Object) b;
 
-	if(_obj == other_object)
-		return TRUE;
-	if(!(_obj && other_object))
-		return FALSE;
-
 	g_assert (_obj->object_key && other_object->object_key);
 
 	if (!IOP_ObjectKey_equal (_obj->object_key, other_object->object_key))
@@ -68,6 +65,7 @@ ORBit_register_objref (CORBA_Object obj)
 		objrefs = g_hash_table_new (
 			g_CORBA_Object_hash, g_CORBA_Object_equal);
 
+	g_assert (obj->object_key != NULL);
 	g_assert (obj->profile_list != NULL);
 
 	g_hash_table_insert (objrefs, obj, obj);
@@ -76,13 +74,7 @@ ORBit_register_objref (CORBA_Object obj)
 static CORBA_Object
 ORBit_lookup_objref (CORBA_Object obj)
 {
-	if (!objrefs) {
-		objrefs = g_hash_table_new (
-			g_CORBA_Object_hash, g_CORBA_Object_equal);
-		return NULL;
-	}
-
-	if (!obj->profile_list)
+	if (!objrefs || !obj->profile_list)
 		return NULL;
 
 	return g_hash_table_lookup (objrefs, obj);
@@ -96,22 +88,20 @@ CORBA_Object_release_cb (ORBit_RootObject robj)
 	if (obj->profile_list)
 		g_hash_table_remove (objrefs, obj);
 
-	if (obj->connection) {
-/*    g_warning("Release object '%p's connection", obj); */
-		giop_connection_close (obj->connection);
-		g_object_unref (G_OBJECT (obj->connection));
-	}
-
-	g_free (obj->type_id);
 	ORBit_free_T (obj->object_key);
 
-	IOP_delete_profiles (&obj->profile_list);
-	IOP_delete_profiles (&obj->forward_locations);
+	IOP_delete_profiles (obj->orb, &obj->profile_list);
+	IOP_delete_profiles (obj->orb, &obj->forward_locations);
+
+	if (obj->adaptor_obj) {
+		obj->adaptor_obj->objref = NULL;
 
-	if (obj->adaptor_obj)
 		ORBit_RootObject_release_T (obj->adaptor_obj);
+	}
 
-	g_free (obj);
+	giop_connection_unref (obj->connection);
+
+	p_free (obj, struct CORBA_Object_type);
 }
 
 static ORBit_RootObject_Interface objref_if = {
@@ -120,7 +110,7 @@ static ORBit_RootObject_Interface objref_if = {
 };
 
 CORBA_Object
-ORBit_objref_new (CORBA_ORB orb, const char *type_id)
+ORBit_objref_new (CORBA_ORB orb, GQuark type_id)
 {
 	CORBA_Object retval;
 
@@ -128,7 +118,7 @@ ORBit_objref_new (CORBA_ORB orb, const char *type_id)
 
 	ORBit_RootObject_init ((ORBit_RootObject) retval, &objref_if);
 
-	retval->type_id = g_strdup (type_id);
+	retval->type_qid = type_id;
 	retval->orb = orb;
 
 	return retval;
@@ -142,7 +132,7 @@ ORBit_objref_find (CORBA_ORB   orb,
 	CORBA_Object retval = CORBA_OBJECT_NIL;
 	struct CORBA_Object_type fakeme = {{0}};
 
-	fakeme.type_id = (char *)type_id;
+	fakeme.type_qid = g_quark_from_string (type_id);
 	fakeme.profile_list = profiles;
 	fakeme.object_key = IOP_profiles_sync_objkey (profiles);
 
@@ -150,62 +140,64 @@ ORBit_objref_find (CORBA_ORB   orb,
 
 	retval = ORBit_lookup_objref (&fakeme);
 
+#ifdef DEBUG
+	g_warning ("Lookup '%s' (%p) == %p", type_id, profiles, retval);
+	{
+		GSList *l;
+		g_print ("Profiles: ");
+		for (l = profiles; l; l = l->next)
+			g_print ("%s", IOP_profile_dump (&fakeme, l->data));
+		g_print ("\n");
+	}
+#endif
+
 	if (!retval) {
-		retval               = ORBit_objref_new (orb, type_id);
+		retval = ORBit_objref_new (orb, fakeme.type_qid);
 		retval->profile_list = profiles;
 		retval->object_key   = fakeme.object_key;
 		ORBit_register_objref (retval);
-	} else
-		IOP_delete_profiles (&profiles);
+	} else {
+		ORBit_free_T (fakeme.object_key);
+		IOP_delete_profiles (orb, &profiles);
+	}
 
-	retval = ORBit_RootObject_duplicate (retval);
+	retval = ORBit_RootObject_duplicate_T (retval);
 
 	LINC_MUTEX_UNLOCK (ORBit_RootObject_lifecycle_lock);
 
 	return retval;
 }
 
-void
-ORBit_start_servers (CORBA_ORB orb)
+/**
+ * ORBit_objref_get_proxy:
+ * @obj: the local object
+ * 
+ *  Creates a 'remote' alike object for the fully local
+ * object reference, so that we can deal with it
+ * asynchronously. Defeats the orb cleverness to stop this,
+ * doesn't register in the global object / profile hash.
+ *
+ *  You almost certainly don't want to use this routine.
+ *
+ * FIXME: we should have an ORB wide 'socketpair' profile
+ * that is private for purely in-proc local loopback support.
+ * 
+ * Return value: a proxy object
+ **/
+CORBA_Object
+ORBit_objref_get_proxy (CORBA_Object obj)
 {
-  LINCProtocolInfo *info;
+	CORBA_Object iobj;
 
-  for(info = linc_protocol_all(); info->name; info++)
-    {
-      GIOPServer *server;
-      LINCConnectionOptions options = 0;
+	if (!obj->profile_list)
+		IOP_generate_profiles (obj);
 
-      if (!ORBit_proto_use (info->name))
-        continue;
+	/* We need a pseudo-remote reference */
+	iobj = ORBit_objref_new (obj->orb, obj->type_qid);
+	iobj->profile_list = IOP_profiles_copy (obj->profile_list);
+	iobj->object_key = IOP_ObjectKey_copy (obj->object_key);
 
-#ifndef ORBIT_THREADED
-      options |= LINC_CONNECTION_NONBLOCKING;
-#endif
-      server = giop_server_new(orb->default_giop_version,
-                               info->name, NULL, NULL,
-                               options, orb);
-      if(server)
-        {
-          orb->servers = g_slist_prepend(orb->servers, server);
-          if(!(info->flags & LINC_PROTOCOL_SECURE))
-            {
-              if (!ORBit_proto_use ("SSL"))
-                continue;
-
-              server = giop_server_new(orb->default_giop_version, info->name,
-                                       NULL, NULL, LINC_CONNECTION_SSL, orb);
-              if(server)
-                orb->servers = g_slist_prepend(orb->servers, server);
-            }
-#ifdef DEBUG
-          fprintf (stderr, "ORB created giop server '%s'\n", info->name);
-#endif
-        }
-#ifdef DEBUG
-      else
-        fprintf (stderr, "ORB failed to create giop server '%s'\n", info->name);
-#endif
-    }
+	return ORBit_RootObject_duplicate (iobj);
 }
 
 static gboolean
@@ -222,9 +214,11 @@ ORBit_try_connection (CORBA_Object obj)
 			break;
 
 		case LINC_DISCONNECTED:
-			if (obj->connection)
-				g_object_unref (G_OBJECT (obj->connection));
-			obj->connection = NULL;
+			if (obj->connection) {
+				giop_connection_unref (obj->connection);
+				obj->connection = NULL;
+			}
+
 			return FALSE;
 			break;
 		}
@@ -234,7 +228,8 @@ ORBit_try_connection (CORBA_Object obj)
 }
 
 GIOPConnection *
-ORBit_object_get_connection (CORBA_Object obj) {
+ORBit_object_get_connection (CORBA_Object obj)
+{
 	GSList *plist, *cur;
 	char tbuf[20];
 	/* Information we have to come up with */
@@ -251,8 +246,7 @@ ORBit_object_get_connection (CORBA_Object obj) {
 	if (!obj->forward_locations) {
 		plist = obj->profile_list;
 		objkey = obj->object_key;
-	}
-	else {
+	} else {
 		plist = obj->forward_locations;
 		objkey = IOP_profiles_sync_objkey (plist);
 	}
@@ -292,7 +286,7 @@ ORBit_handle_location_forward (GIOPRecvBuffer *buf,
 	if (ORBit_demarshal_IOR (obj->orb, buf, NULL, &profiles))
 		goto out;
 
-	IOP_delete_profiles (&obj->forward_locations);
+	IOP_delete_profiles (obj->orb, &obj->forward_locations);
 
 	obj->forward_locations = profiles;
 
@@ -327,45 +321,56 @@ CORBA_Object_duplicate (CORBA_Object       obj,
 }
 
 void
-CORBA_Object_release(CORBA_Object _obj, CORBA_Environment * ev)
+CORBA_Object_release (CORBA_Object        obj,
+		      CORBA_Environment *ev)
 {
-  ORBit_RootObject_release(_obj);
+	ORBit_RootObject_release (obj);
 }
 
 CORBA_boolean
-CORBA_Object_non_existent(CORBA_Object       obj,
-			  CORBA_Environment *ev)
+CORBA_Object_non_existent (CORBA_Object       obj,
+			   CORBA_Environment *ev)
 {
-  ORBit_OAObject adaptor_obj;
+	ORBit_OAObject adaptor_obj;
 
-  if(obj == CORBA_OBJECT_NIL)
-    return TRUE;
+	if (obj == CORBA_OBJECT_NIL)
+		return TRUE;
 
-  adaptor_obj = obj->adaptor_obj;
-  if (adaptor_obj && adaptor_obj->interface->is_active (adaptor_obj))
-    return FALSE;
+	adaptor_obj = obj->adaptor_obj;
+	if (adaptor_obj && adaptor_obj->interface->is_active (adaptor_obj))
+		return FALSE;
 
-  return ORBit_object_get_connection(obj)?CORBA_FALSE:CORBA_TRUE;
+	return ORBit_object_get_connection (obj) ? CORBA_FALSE : CORBA_TRUE;
 }
 
 CORBA_boolean
-CORBA_Object_is_equivalent(CORBA_Object _obj,
-			   CORBA_Object other_object,
-			   CORBA_Environment * ev)
+CORBA_Object_is_equivalent (CORBA_Object       obj,
+			    CORBA_Object       other_object,
+			    CORBA_Environment *ev)
 {
-  return g_CORBA_Object_equal(_obj, other_object);
+	if (obj == other_object)
+		return TRUE;
+
+	if (!(obj && other_object))
+		return FALSE;
+
+	/* local - have never been exposed; but not both local */
+	if (!obj->object_key || !other_object->object_key)
+		return FALSE;
+
+	return g_CORBA_Object_equal (obj, other_object);
 }
 
 CORBA_unsigned_long
-CORBA_Object_hash(CORBA_Object _obj,
-		  const CORBA_unsigned_long maximum,
-		  CORBA_Environment * ev)
+CORBA_Object_hash (CORBA_Object              obj,
+		   const CORBA_unsigned_long maximum,
+		   CORBA_Environment        *ev)
 {
-  CORBA_unsigned_long retval;
+	CORBA_unsigned_long retval;
 
-  retval = g_CORBA_Object_hash(_obj);
+	retval = g_CORBA_Object_hash (obj);
 
-  return maximum?(retval%maximum):retval;
+	return maximum ? (retval % maximum) : retval;
 }
 
 void
@@ -378,96 +383,88 @@ CORBA_Object_create_request (CORBA_Object         _obj,
 			     const CORBA_Flags    req_flag,
 			     CORBA_Environment   *ev)
 {
-  CORBA_exception_set_system(ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+	CORBA_exception_set_system (ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
 }
 
 CORBA_Policy
-CORBA_Object_get_policy(CORBA_Object _obj,
-			const CORBA_PolicyType policy_type,
-			CORBA_Environment * ev)
+CORBA_Object_get_policy (CORBA_Object           obj,
+			 const CORBA_PolicyType policy_type,
+			 CORBA_Environment     *ev)
 {
-  CORBA_exception_set_system(ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
-  return CORBA_OBJECT_NIL;
+	CORBA_exception_set_system (ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+	return CORBA_OBJECT_NIL;
 }
 
 CORBA_DomainManagersList *
-CORBA_Object_get_domain_managers(CORBA_Object _obj,
-				 CORBA_Environment * ev)
+CORBA_Object_get_domain_managers (CORBA_Object       obj,
+				  CORBA_Environment *ev)
 {
-  CORBA_exception_set_system(ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
-  return NULL;
+	CORBA_exception_set_system (ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+	return NULL;
 }
 
 CORBA_Object
-CORBA_Object_set_policy_overrides(CORBA_Object _obj,
-				  const CORBA_PolicyList *policies,
-				  const CORBA_SetOverrideType set_add,
-				  CORBA_Environment * ev)
+CORBA_Object_set_policy_overrides (CORBA_Object                obj,
+				   const CORBA_PolicyList     *policies,
+				   const CORBA_SetOverrideType set_add,
+				   CORBA_Environment          *ev)
 {
-  CORBA_exception_set_system(ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
-  return CORBA_OBJECT_NIL;
+	CORBA_exception_set_system (ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+	return CORBA_OBJECT_NIL;
 }
 
 void
-ORBit_marshal_object(GIOPSendBuffer *buf, CORBA_Object obj)
+ORBit_marshal_object (GIOPSendBuffer *buf, CORBA_Object obj)
 {
-  CORBA_unsigned_long type_len, num_profiles;
-  GSList *cur;
-  char *typeid;
+	GSList             *cur;
+	const char         *typeid;
+	CORBA_unsigned_long num_profiles;
 
-  if(obj)
-    typeid = obj->type_id;
-  else
-    typeid = "";
-  type_len = strlen(typeid) + 1;
-  giop_send_buffer_append_aligned (buf, &type_len, 4);
-  giop_send_buffer_append(buf, typeid, type_len);
-  if(obj)
-    {
-      if ( obj->profile_list == NULL )
-	  IOP_generate_profiles( obj );
-      num_profiles = g_slist_length(obj->profile_list);
-      g_assert (num_profiles > 0);
-    }
-  else
-    num_profiles = 0;
-  giop_send_buffer_append_aligned(buf, &num_profiles, 4);
+	if (obj)
+		typeid = g_quark_to_string (obj->type_qid);
+	else
+		typeid = "";
+
+	giop_send_buffer_append_string (buf, typeid);
+
+	if (obj) {
+		if (!obj->profile_list)
+			IOP_generate_profiles (obj);
+		num_profiles = g_slist_length (obj->profile_list);
+		g_assert (num_profiles > 0);
+	} else
+		num_profiles = 0;
+	giop_send_buffer_append_aligned (buf, &num_profiles, 4);
 
 #ifdef DEBUG
-  fprintf (stderr, "Marshal object '%p'\n", obj);
+	fprintf (stderr, "Marshal object '%p'\n", obj);
 #endif
-  if(obj)
-    for(cur = obj->profile_list; cur; cur = cur->next)
-      {
+	if (obj)
+		for (cur = obj->profile_list; cur; cur = cur->next) {
 #ifdef DEBUG
-        fprintf (stderr, "%s\n", IOP_profile_dump (obj, cur->data));
+			fprintf (stderr, "%s\n", IOP_profile_dump (obj, cur->data));
 #endif
-	IOP_profile_marshal(obj, buf, cur->data);
-      }
+			IOP_profile_marshal (obj, buf, cur->data);
+		}
 }
 
 gboolean
-ORBit_demarshal_object(CORBA_Object *obj, GIOPRecvBuffer *buf,
-		       CORBA_ORB orb)
+ORBit_demarshal_object (CORBA_Object   *obj,
+			GIOPRecvBuffer *buf,
+			CORBA_ORB       orb)
 {
-  GSList *profiles;
-  char *type_id;
+	char   *type_id;
+	GSList *profiles;
 
-  if(ORBit_demarshal_IOR(orb, buf, &type_id, &profiles))
-    return TRUE;
+	if (ORBit_demarshal_IOR (orb, buf, &type_id, &profiles))
+		return TRUE;
 
-  if(type_id)
-    *obj = ORBit_objref_find(orb, type_id, profiles);
-  else
-    *obj = CORBA_OBJECT_NIL;
+	if (type_id)
+		*obj = ORBit_objref_find (orb, type_id, profiles);
+	else
+		*obj = CORBA_OBJECT_NIL;
 
-  return FALSE;
-}
-
-gpointer
-CORBA_Object__freekids(gpointer mem, gpointer dat)
-{
-  return mem + sizeof(CORBA_Object);
+	return FALSE;
 }
 
 CORBA_boolean
@@ -475,16 +472,24 @@ CORBA_Object_is_a (CORBA_Object       obj,
 		   const CORBA_char  *logical_type_id,
 		   CORBA_Environment *ev)
 {
+	static GQuark  corba_object_quark = 0;
 	CORBA_boolean  retval;
 	gpointer       args[] = { (gpointer *)&logical_type_id };
+	GQuark         logical_type_quark;
 
-	if (strcmp (logical_type_id, "IDL:CORBA/Object:1.0") == 0)
+	if (!corba_object_quark)
+		corba_object_quark = g_quark_from_static_string (
+			"IDL:CORBA/Object:1.0");
+
+	logical_type_quark = g_quark_from_string (logical_type_id);
+
+	if (logical_type_quark == corba_object_quark)
 		return CORBA_TRUE;
 
 	if (!obj)
 		return CORBA_FALSE;
 
-	if (strcmp (logical_type_id, obj->type_id) == 0)
+	if (logical_type_quark == obj->type_qid)
 		return CORBA_TRUE;
 
 	ORBit_small_invoke_stub (
