@@ -1,7 +1,8 @@
-#include <orbit/orbit.h>
+#include <config.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <orbit/orbit.h>
 
 #include "iop-profiles.h"
 #include "orb-core-private.h"
@@ -13,18 +14,13 @@
    used the wire structures more intelligently we could store
    everything far more sensibly */
 
-/*
- * common set of profiles for Ojbect Adaptor generated refs
- */
-static GSList *common_profiles = NULL;
-
 static void IOP_profile_free (IOP_Profile_info *p);
 
 #ifdef LINC_SSL_SUPPORT
 static IOP_Component_info *
-IOP_component_find(GSList *list, IOP_ComponentId type, GSList **pos)
+IOP_component_find (GSList *list, IOP_ComponentId type, GSList **pos)
 {
-  for(; list; list = list->next)
+  for (; list; list = list->next)
     {
       IOP_Component_info *pi = list->data;
       if(pi->component_type == type)
@@ -48,7 +44,7 @@ IOP_ObjectKey_dump (ORBit_ObjectKey *objkey)
 }
 
 G_GNUC_UNUSED gchar * 
-IOP_profile_dump(CORBA_Object obj, gpointer p)
+IOP_profile_dump (CORBA_Object obj, gpointer p)
 {
 	IOP_ProfileId t;
 	char         *key = NULL;
@@ -447,179 +443,172 @@ IOP_profile_hash(gpointer item, gpointer data)
 }
 
 void
-IOP_delete_profiles (GSList **profiles)
+IOP_delete_profiles (CORBA_ORB orb,
+		     GSList  **profiles)
 {
-  if (profiles && *profiles && (*profiles != common_profiles)) {
-    g_slist_foreach (*profiles, (GFunc)IOP_profile_free, NULL);
-    g_slist_free (*profiles);
-    *profiles = NULL;
-  }
+	if (!profiles || !*profiles)
+		return;
+
+	if (orb && *profiles == orb->profiles)
+		return;
+
+	g_slist_foreach (*profiles, (GFunc)IOP_profile_free, NULL);
+	g_slist_free (*profiles);
+	*profiles = NULL;
+}
+
+GSList *
+IOP_start_profiles (CORBA_ORB orb)
+{
+	GSList  *l;
+	GSList  *common_profiles = NULL;
+	gboolean need_objkey_component = FALSE;
+	IOP_TAG_MULTIPLE_COMPONENTS_info *mci = NULL;
+	IOP_TAG_ORBIT_SPECIFIC_info      *osi = NULL;
+	IOP_TAG_INTERNET_IOP_info        *iiop = NULL;
+
+	for (l = orb->servers ; l != NULL ; l = l->next) {
+		LINCServer *serv = l->data;
+		gboolean   ipv4, ipv6, uds, ssl;
+
+		ipv4 = !strcmp (serv->proto->name, "IPv4");
+		ipv6 = !strcmp (serv->proto->name, "IPv6");
+		uds  = !strcmp (serv->proto->name, "UNIX");
+
+		ssl  = (serv->create_options & LINC_CONNECTION_SSL);
+
+		if (!osi && (uds || (ipv6 && !ssl))) {
+			osi = g_new0 (IOP_TAG_ORBIT_SPECIFIC_info, 1);
+			osi->parent.profile_type = IOP_TAG_ORBIT_SPECIFIC;
+		}
+
+		if (uds && !osi->unix_sock_path)
+			osi->unix_sock_path = g_strdup (serv->local_serv_info);
+
+		if (ipv6 && !ssl)
+			osi->ipv6_port = atoi (serv->local_serv_info);
+
+		if (ipv4) {
+			if (!iiop) {
+				iiop = g_new0 (IOP_TAG_INTERNET_IOP_info, 1);
+				iiop->host = g_strdup (serv->local_host_info);
+				common_profiles = g_slist_append (common_profiles, iiop);
+			}
+
+			if (ssl) {
+				IOP_TAG_SSL_SEC_TRANS_info *sslsec;
+
+				sslsec = g_new0 (IOP_TAG_SSL_SEC_TRANS_info, 1);
+				sslsec->parent.component_type = IOP_TAG_SSL_SEC_TRANS;
+				/* integrity & confidentiality */
+				sslsec->target_supports = sslsec->target_requires = 2|4;
+				sslsec->port = atoi (serv->local_serv_info);
+				iiop->components = g_slist_append (
+					iiop->components, sslsec);
+			} else {
+				g_assert (!iiop->port);
+				iiop->port = atoi (serv->local_serv_info);
+				iiop->iiop_version = orb->default_giop_version;
+			}
+		} else {
+			GSList *l2;
+			IOP_TAG_GENERIC_IOP_info *giop;
+
+			for (giop = NULL, l2 = common_profiles; l2; l2 = l2->next) {
+				IOP_TAG_GENERIC_IOP_info *giopt;
+
+				giopt = l2->data;
+				if (giopt->parent.profile_type == IOP_TAG_GENERIC_IOP &&
+				    strcmp (giopt->proto, serv->proto->name)) {
+					giop = giopt;
+					break;
+				}
+			}
+
+			if (!giop) {
+				giop = g_new0 (IOP_TAG_GENERIC_IOP_info, 1);
+				giop->parent.profile_type = IOP_TAG_GENERIC_IOP;
+				giop->iiop_version = orb->default_giop_version;
+				giop->proto = g_strdup (serv->proto->name);
+				giop->host = g_strdup (serv->local_host_info);
+				common_profiles = g_slist_append (common_profiles, giop);
+			}
+
+			if (ssl) {
+				IOP_TAG_GENERIC_SSL_SEC_TRANS_info *sslsec;
+
+				sslsec = g_new0 (IOP_TAG_GENERIC_SSL_SEC_TRANS_info, 1);
+				sslsec->parent.component_type = IOP_TAG_GENERIC_SSL_SEC_TRANS;
+				sslsec->service = g_strdup (serv->local_serv_info);
+				giop->components = g_slist_append (giop->components, sslsec);
+			} else {
+				g_assert (!giop->service);
+				giop->service = g_strdup (serv->local_serv_info);
+			}
+		}
+		need_objkey_component = TRUE;
+	}
+
+	if (osi)
+		common_profiles = g_slist_append (common_profiles, osi);
+
+	/* We always create this to marshal the TAG_CODE_SET component */
+	mci = g_new0 (IOP_TAG_MULTIPLE_COMPONENTS_info, 1);
+	mci->parent.profile_type = IOP_TAG_MULTIPLE_COMPONENTS;
+
+	if (need_objkey_component) {
+		IOP_TAG_COMPLETE_OBJECT_KEY_info *coki;
+
+		coki = g_new0 (IOP_TAG_COMPLETE_OBJECT_KEY_info, 1);
+		coki->parent.component_type = IOP_TAG_COMPLETE_OBJECT_KEY;
+		mci->components = g_slist_append (mci->components, coki);
+	}
+
+	{
+		IOP_TAG_CODE_SETS_info *csets;
+
+		csets = g_new0 (IOP_TAG_CODE_SETS_info, 1);
+		csets->parent.component_type = IOP_TAG_CODE_SETS;
+		mci->components = g_slist_append (mci->components, csets);
+	}
+
+	return g_slist_append (common_profiles, mci);
+}
+
+void
+IOP_shutdown_profiles (GSList *profiles)
+{
+	g_slist_foreach (profiles, (GFunc)IOP_profile_free, NULL);
+	g_slist_free (profiles);
 }
 
 void
 IOP_generate_profiles (CORBA_Object obj)
 {
-  IOP_TAG_MULTIPLE_COMPONENTS_info *mci = NULL;
-  IOP_TAG_ORBIT_SPECIFIC_info      *osi = NULL;
-  IOP_TAG_INTERNET_IOP_info        *iiop = NULL;
-  gboolean                          need_objkey_component;
-  ORBit_OAObject                    adaptor_obj = obj->adaptor_obj;
-  GSList                           *ltmp;
+	CORBA_ORB      orb;
+	ORBit_OAObject adaptor_obj;
 
-  g_assert( obj && (obj->profile_list == NULL) );
+	g_assert (obj && (obj->profile_list == NULL) && obj->orb);
 
-  /*
-   * no need to have any listening sockets until now.
-   * if the ORB has been shutdown and restarted,
-   * the profiles must be regenerated.
-   */
-  if (!obj->orb->servers) {
-    ORBit_start_servers (obj->orb);
-    if (common_profiles) {
-      g_slist_foreach (common_profiles, (GFunc)IOP_profile_free, NULL);
-      g_slist_free (common_profiles);
-      common_profiles = NULL;
-    }
-  }
+	orb = obj->orb;
+	adaptor_obj = obj->adaptor_obj;
 
-  if (!obj->object_key && adaptor_obj)
-    obj->object_key = ORBit_OAObject_object_to_objkey (adaptor_obj);
+	/*
+	 * no need to have any listening sockets until now.
+	 * if the ORB has been shutdown and restarted,
+	 * the profiles must be regenerated.
+	 */
+	if (!orb->servers)
+		ORBit_ORB_start_servers (orb);
 
-  /* share the profiles between everyone. */
-  if (common_profiles) {
-    obj->profile_list = common_profiles;
-    return;
-  }
+	if (!obj->object_key && adaptor_obj)
+		obj->object_key = ORBit_OAObject_object_to_objkey (adaptor_obj);
 
-  need_objkey_component = FALSE;
-  for(ltmp = obj->orb->servers ; ltmp != NULL ; ltmp = ltmp->next)
-    {
-      LINCServer *serv = ltmp->data;
-      gboolean   ipv4, ipv6, uds, ssl;
+	obj->profile_list = orb->profiles;
 
-      ipv4 = (strcmp(serv->proto->name, "IPv4") == 0)     ? TRUE : FALSE;
-      ipv6 = (strcmp(serv->proto->name, "IPv6") == 0)     ? TRUE : FALSE;
-      uds  = (strcmp(serv->proto->name, "UNIX") == 0)     ? TRUE : FALSE;
-      ssl  = (serv->create_options & LINC_CONNECTION_SSL) ? TRUE : FALSE;
-
-      if( osi == NULL && (uds || (ipv6 && !ssl)) )
-        {
-          osi = g_new0(IOP_TAG_ORBIT_SPECIFIC_info, 1);
-          osi->parent.profile_type = IOP_TAG_ORBIT_SPECIFIC;
-        }
-
-      if(uds && osi->unix_sock_path == NULL )
-        osi->unix_sock_path = g_strdup(serv->local_serv_info);
-
-      if(ipv6 && !ssl)
-        osi->ipv6_port = atoi(serv->local_serv_info);
-
-      if(ipv4)
-        {
-          if(iiop == NULL)
-            {
-              iiop = g_new0(IOP_TAG_INTERNET_IOP_info, 1);
-              iiop->host = g_strdup(serv->local_host_info);
-              common_profiles = g_slist_append(common_profiles, iiop);
-            }
-
-          if(ssl)
-            {
-              IOP_TAG_SSL_SEC_TRANS_info *sslsec;
-
-              sslsec = g_new0(IOP_TAG_SSL_SEC_TRANS_info, 1);
-              sslsec->parent.component_type = IOP_TAG_SSL_SEC_TRANS;
-              /* integrity & confidentiality */
-              sslsec->target_supports = sslsec->target_requires = 2|4;
-              sslsec->port = atoi(serv->local_serv_info);
-              iiop->components = g_slist_append(iiop->components, sslsec);
-            }
-          else
-            {
-              g_assert(!iiop->port);
-              iiop->port = atoi(serv->local_serv_info);
-              iiop->iiop_version = obj->orb->default_giop_version;
-            }
-        }
-      else
-        {
-          GSList *ltmp2;
-          IOP_TAG_GENERIC_IOP_info *giop;
-
-          for(giop = NULL, ltmp2 = common_profiles; ltmp2; ltmp2 = ltmp2->next)
-            {
-              IOP_TAG_GENERIC_IOP_info *giopt;
-
-              giopt = ltmp2->data;
-              if(giopt->parent.profile_type == IOP_TAG_GENERIC_IOP
-                 && strcmp(giopt->proto, serv->proto->name))
-                {
-                  giop = giopt;
-                  break;
-                }
-            }
-
-          if(giop == NULL)
-            {
-              giop = g_new0(IOP_TAG_GENERIC_IOP_info, 1);
-              giop->parent.profile_type = IOP_TAG_GENERIC_IOP;
-              giop->iiop_version = obj->orb->default_giop_version;
-              giop->proto = g_strdup(serv->proto->name);
-              giop->host = g_strdup(serv->local_host_info);
-              common_profiles = g_slist_append(common_profiles, giop);
-            }
-
-          if(ssl)
-            {
-              IOP_TAG_GENERIC_SSL_SEC_TRANS_info *sslsec;
-
-              sslsec = g_new0(IOP_TAG_GENERIC_SSL_SEC_TRANS_info, 1);
-              sslsec->parent.component_type = IOP_TAG_GENERIC_SSL_SEC_TRANS;
-              sslsec->service = g_strdup(serv->local_serv_info);
-              giop->components = g_slist_append(giop->components, sslsec);
-            }
-          else
-            {
-              g_assert(!giop->service);
-              giop->service = g_strdup(serv->local_serv_info);
-            }
-        }
-      need_objkey_component = TRUE;
-    }
-
-  if (osi)
-    common_profiles = g_slist_append (common_profiles, osi);
-
-  /* We always create this to marshal the TAG_CODE_SET component */
-  mci = g_new0 (IOP_TAG_MULTIPLE_COMPONENTS_info, 1);
-  mci->parent.profile_type = IOP_TAG_MULTIPLE_COMPONENTS;
-
-  if (need_objkey_component) {
-    IOP_TAG_COMPLETE_OBJECT_KEY_info *coki;
-
-    coki = g_new0 (IOP_TAG_COMPLETE_OBJECT_KEY_info, 1);
-    coki->parent.component_type = IOP_TAG_COMPLETE_OBJECT_KEY;
-    mci->components = g_slist_append (mci->components, coki);
-  }
-
-  {
-    IOP_TAG_CODE_SETS_info *csets;
-
-    csets = g_new0 (IOP_TAG_CODE_SETS_info, 1);
-    csets->parent.component_type = IOP_TAG_CODE_SETS;
-    mci->components = g_slist_append (mci->components, csets);
-  }
-
-  common_profiles = g_slist_append (common_profiles, mci);
-
-  obj->profile_list = common_profiles;
-  ORBit_register_objref (obj);
+	ORBit_register_objref (obj);
 }
 
-/*
- * 'freedom' routines.
- */
-
 static void
 IOP_component_free (IOP_Component_info *c)
 {
@@ -731,10 +720,6 @@ IOP_profile_free (IOP_Profile_info *p)
 	g_free (p);
 }
 
-/*
- * marshalling routines.
- */
-
 static void
 IOP_ObjectKey_marshal (CORBA_Object                obj,
 		       GIOPSendBuffer             *buf,
@@ -746,32 +731,34 @@ IOP_ObjectKey_marshal (CORBA_Object                obj,
 }
 
 static void
-IOP_TAG_GENERIC_SSL_SEC_TRANS_marshal(CORBA_Object obj, GIOPSendBuffer *buf,
-				      IOP_Component_info *ci)
+IOP_TAG_GENERIC_SSL_SEC_TRANS_marshal (CORBA_Object        obj,
+				       GIOPSendBuffer     *buf,
+				       IOP_Component_info *ci)
 {
-  CORBA_unsigned_long len;
-  IOP_TAG_GENERIC_SSL_SEC_TRANS_info *ssli = (IOP_TAG_GENERIC_SSL_SEC_TRANS_info *)ci;
+	IOP_TAG_GENERIC_SSL_SEC_TRANS_info *ssli;
+	
+	ssli = (IOP_TAG_GENERIC_SSL_SEC_TRANS_info *)ci;
 
-  len = strlen(ssli->service) + 1;
-  giop_send_buffer_append_aligned(buf, &len, 4);
-  giop_send_buffer_append(buf, ssli->service, 4);
+	giop_send_buffer_append_string (buf, ssli->service);
 }
 
 static void
-IOP_TAG_SSL_SEC_TRANS_marshal(CORBA_Object obj, GIOPSendBuffer *buf,
+IOP_TAG_SSL_SEC_TRANS_marshal(CORBA_Object        obj,
+			      GIOPSendBuffer     *buf,
 			      IOP_Component_info *ci)
 {
-  IOP_TAG_SSL_SEC_TRANS_info *ssli = (IOP_TAG_SSL_SEC_TRANS_info *) ci;
+	IOP_TAG_SSL_SEC_TRANS_info *ssli = (IOP_TAG_SSL_SEC_TRANS_info *) ci;
 
-  giop_send_buffer_align (buf, 4);
-  giop_send_buffer_append (buf, &ssli->target_supports, 10);
+	giop_send_buffer_align (buf, 4);
+	giop_send_buffer_append (buf, &ssli->target_supports, 10);
 }
 
 static void
-IOP_TAG_COMPLETE_OBJECT_KEY_marshal(CORBA_Object obj, GIOPSendBuffer *buf,
+IOP_TAG_COMPLETE_OBJECT_KEY_marshal (CORBA_Object       obj,
+				     GIOPSendBuffer    *buf,
 				    IOP_Component_info *ci)
 {
-  IOP_ObjectKey_marshal (obj, buf, obj->object_key);
+	IOP_ObjectKey_marshal (obj, buf, obj->object_key);
 }
 
 static void
@@ -806,18 +793,19 @@ IOP_TAG_CODE_SETS_marshal(CORBA_Object obj, GIOPSendBuffer *buf,
 }
 
 static void
-IOP_UnknownComponent_marshal(CORBA_Object obj, GIOPSendBuffer *buf, 
-			     IOP_Component_info *ci)
+IOP_UnknownComponent_marshal (CORBA_Object        obj,
+			      GIOPSendBuffer     *buf, 
+			      IOP_Component_info *ci)
 {
-  IOP_UnknownComponent_info *uci = (IOP_UnknownComponent_info *)ci;
+	IOP_UnknownComponent_info *uci = (IOP_UnknownComponent_info *) ci;
 
-  giop_send_buffer_append(buf, uci->data._buffer, uci->data._length);
+	giop_send_buffer_append (buf, uci->data._buffer, uci->data._length);
 }
 
 static void
-IOP_components_marshal (CORBA_Object obj, 
+IOP_components_marshal (CORBA_Object    obj, 
 			GIOPSendBuffer *buf,
-			GSList *components)
+			GSList         *components)
 {
 	CORBA_unsigned_long  len;
 	GSList              *cur;
@@ -875,16 +863,13 @@ IOP_TAG_INTERNET_IOP_marshal (CORBA_Object      obj,
 			      IOP_Profile_info *profile)
 {
 	IOP_TAG_INTERNET_IOP_info *iiop;
-	CORBA_unsigned_long len;
 
 	iiop = (IOP_TAG_INTERNET_IOP_info *) profile;
 
 	giop_send_buffer_append (
 		buf, giop_version_ids [iiop->iiop_version], 2);
 
-	len = strlen (iiop->host) + 1;
-	giop_send_buffer_append_aligned (buf, &len, 4);
-	giop_send_buffer_append (buf, iiop->host, len);
+	giop_send_buffer_append_string (buf, iiop->host);
 
 	giop_send_buffer_align (buf, 2);
 	giop_send_buffer_append (buf, &iiop->port, 2);
@@ -900,20 +885,12 @@ IOP_TAG_GENERIC_IOP_marshal (CORBA_Object     obj,
 			     IOP_Profile_info *profile)
 {
 	IOP_TAG_GENERIC_IOP_info *giop = (IOP_TAG_GENERIC_IOP_info *) profile;
-	CORBA_unsigned_long       len;
 
 	giop_send_buffer_append (buf, giop_version_ids [giop->iiop_version], 2);
-	len = strlen (giop->proto) + 1;
-	giop_send_buffer_append_aligned (buf, &len, 4);
-	giop_send_buffer_append (buf, giop->proto, len);
 
-	len = strlen (giop->host) + 1;
-	giop_send_buffer_append_aligned (buf, &len, 4);
-	giop_send_buffer_append (buf, giop->host, len);
-
-	len = strlen (giop->service) + 1;
-	giop_send_buffer_append_aligned (buf, &len, 4);
-	giop_send_buffer_append (buf, giop->service, len);
+	giop_send_buffer_append_string (buf, giop->proto);
+	giop_send_buffer_append_string (buf, giop->host);
+	giop_send_buffer_append_string (buf, giop->service);
 
 	IOP_components_marshal (obj, buf, giop->components);
 }
@@ -935,7 +912,6 @@ IOP_TAG_ORBIT_SPECIFIC_marshal (CORBA_Object      obj,
 				GIOPSendBuffer   *buf,
 				IOP_Profile_info *profile)
 {
-	CORBA_unsigned_long len;
 	IOP_TAG_ORBIT_SPECIFIC_info *osi;
 	guchar compat_version[2] = { 1, 2 }; /* for ORBit1 */
 
@@ -943,9 +919,7 @@ IOP_TAG_ORBIT_SPECIFIC_marshal (CORBA_Object      obj,
 
 	giop_send_buffer_append (buf, compat_version, 2);
 
-	len = strlen (osi->unix_sock_path) + 1;
-	giop_send_buffer_append_aligned (buf, &len, 4);
-	giop_send_buffer_append (buf, osi->unix_sock_path, len);
+	giop_send_buffer_append_string (buf, osi->unix_sock_path);
 
 	giop_send_buffer_align (buf, 2);
 	giop_send_buffer_append (buf, &osi->ipv6_port, 2);
@@ -1681,7 +1655,7 @@ ORBit_demarshal_IOR(CORBA_ORB orb, GIOPRecvBuffer *buf,
   return FALSE;
 
  errout:
-  IOP_delete_profiles(&profiles);
+  IOP_delete_profiles (orb, &profiles);
   return TRUE;
 }
 
@@ -1774,6 +1748,7 @@ IOP_TAG_INTERNET_IOP_copy (IOP_TAG_INTERNET_IOP_info       *dest,
 			   const IOP_TAG_INTERNET_IOP_info *src)
 {
 	dest->iiop_version = src->iiop_version;
+	dest->host = g_strdup (src->host);
 	dest->port = src->port;
 	dest->object_key = IOP_ObjectKey_copy (src->object_key);
 	dest->components = IOP_components_copy (src->components);
