@@ -13,6 +13,12 @@
  */
 #undef TRACE_DEBUG
 
+/*
+ * Define this to get microsecond time stamps on
+ * each method invocation, a-la strace -ttt
+ */
+#undef TRACE_TIMING
+
 /*
  * Flip this switch if you want tyeplib
  * related warnings.
@@ -30,6 +36,8 @@ static inline void dprintf (const char *format, ...) { };
 
 #else /* DEBUG */
 
+#include <stdio.h>
+
 #define dprintf(format...) fprintf(stderr, format)
 #define do_giop_dump_send(a) giop_dump_send(a)
 #define do_giop_dump_recv(a) giop_dump_recv(a)
@@ -51,18 +59,38 @@ dump_arg (const ORBit_IArg *a, CORBA_TypeCode tc)
 
 static inline void tprintf (const char *format, ...) { };
 #define tprintf_trace_value(a,b)
+#define tprintf_header(obj,md)
+#define tprintf_timestamp()
+#define tprintf_end_method()
 
 #else /* TRACE_DEBUG */
 
-void     ORBit_trace_objref   (const CORBA_Object   obj);
-void     ORBit_trace_any      (const CORBA_any     *any);
-void     ORBit_trace_typecode (const CORBA_TypeCode tc);
-void     ORBit_trace_value    (gconstpointer       *val,
-                               CORBA_TypeCode       tc);
+#include <stdio.h>
+
+void     ORBit_trace_objref     (const CORBA_Object   obj);
+void     ORBit_trace_any        (const CORBA_any     *any);
+void     ORBit_trace_typecode   (const CORBA_TypeCode tc);
+void     ORBit_trace_value      (gconstpointer       *val,
+				 CORBA_TypeCode       tc);
+void     ORBit_trace_header     (CORBA_Object         object,
+				 ORBit_IMethod       *m_data);
+void     ORBit_trace_end_method (void);
 
 #define tprintf(format...) fprintf(stderr, format)
+#define tprintf_header(obj,md) ORBit_trace_header(obj,md)
 #define tprintf_trace_value(a,b) \
 		ORBit_trace_value ((gconstpointer *)(a), (b))
+#define tprintf_end_method() \
+		ORBit_trace_end_method ()
+
+
+#ifdef TRACE_TIMING
+#  define tprintf_timestamp() \
+		ORBit_trace_timestamp ()
+void     ORBit_trace_timestamp (void);
+#else
+#  define tprintf_timestamp()
+#endif
 
 #endif /* TRACE_DEBUG */
 
