@@ -39,25 +39,7 @@
 gpointer
 ORBit_small_alloc (CORBA_TypeCode tc)
 {
-	size_t			element_size;
-	ORBit_MemPrefix_TypeCode	*pre;
-	gpointer 			mem;
-
-	if ((element_size = ORBit_gather_alloc_info(tc)) == 0)
-		return NULL;
-	
-	mem = ORBit_alloc_core (element_size,
-				ORBIT_MEMHOW_TYPECODE | 1,
-				sizeof(*pre), (gpointer)&pre,
-				/*align*/0);
-
-	memset (mem, 0, element_size);
-
-	ORBIT_MEM_MAGICSET(pre->magic);
-
-	pre->tc = ORBit_RootObject_duplicate(tc);
-	
-	return mem;
+	return ORBit_alloc_by_tc (tc);
 }
 
 gpointer
@@ -215,10 +197,7 @@ ORBit_small_send_user_exception (GIOPSendBuffer     *send_buffer,
 
 		return FALSE;
 	} else {
-		CORBA_unsigned_long len = strlen (ev->_id) + 1;
-
-		giop_send_buffer_append_aligned (send_buffer, &len, sizeof (len));
-		giop_send_buffer_append (send_buffer, ev->_id, len);
+		giop_send_buffer_append_string (send_buffer, ev->_id);
 
 		dprintf ("Returning exception of type '%s'\n", ev->_id);
 
@@ -383,6 +362,7 @@ orbit_small_marshal (CORBA_Object           obj,
 	do_giop_dump_send (send_buffer);
 
 	if (giop_send_buffer_write (send_buffer, cnx)) {
+		g_warning ("Failed to send buffer");
 		giop_recv_list_destroy_queue_entry (mqe);
 		return FALSE;
 	}
@@ -539,6 +519,7 @@ orbit_small_demarshal (CORBA_Object           obj,
 	}
 	
 	giop_recv_buffer_unuse (recv_buffer);
+
 	return MARSHAL_CLEAN;
 
  msg_exception:
@@ -639,7 +620,7 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	};
 
  clean_out:
-	tprintf ("\n");
+	tprintf_end_method ();
 	return;
 
  system_exception:
@@ -669,7 +650,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 
 	do_giop_dump_recv (recv_buffer);
 
-	orb = adaptor_obj->objref->orb;
+	orb = ((ORBit_POAObject)adaptor_obj)->poa->orb;
 
 	has_context = (m_data->contexts._length > 0);
 
@@ -687,7 +668,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			break;
 		case STRUCT_UNION_TYPES:
 			if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
-				retval = ORBit_alloc_tcval (tc, 1);
+				retval = ORBit_alloc_by_tc (tc);
 				break;
 			} /* drop through */
 		default:
@@ -759,7 +740,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 			case STRUCT_UNION_TYPES:
 			case CORBA_tk_array:
 				if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
-					scratch [i] = ORBit_alloc_tcval (tc, 1);
+					scratch [i] = ORBit_alloc_by_tc (tc);
 					break;
 				} /* drop through */
 			default:
@@ -918,7 +899,7 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 	}
 
  clean_out:
-	tprintf ("\n");
+	tprintf_end_method ();
 
 	for (i = 0; i < m_data->arguments._length; i++) {
 		ORBit_IArg *a = &m_data->arguments._buffer [i];
@@ -1044,7 +1025,7 @@ ORBit_small_demarshal_async (ORBitAsyncQueueEntry *aqe,
 				    aqe->completion_status);
 
  clean_out:
-	tprintf ("\n");
+	tprintf_end_method ();
 }
 
 static void
@@ -1135,7 +1116,7 @@ ORBit_small_invoke_async (CORBA_Object         obj,
 	aqe->m_data = /* ORBit_RootObject_duplicate */ (m_data);
 
  clean_out:
-	tprintf ("\n");
+	tprintf_end_method ();
 	return;
 
  system_exception:
@@ -1213,8 +1194,12 @@ ORBit_small_listen_for_broken (CORBA_Object obj,
 {
 	ORBitConnectionStatus ret;
 
-	if (ORBit_small_get_servant (obj))
+	if (!obj)
+		ret = ORBIT_CONNECTION_DISCONNECTED;
+
+	else if (ORBit_small_get_servant (obj))
 		ret = ORBIT_CONNECTION_IN_PROC;
+
 	else {
 		GIOPConnection *connection;
 
@@ -1237,8 +1222,12 @@ ORBit_small_unlisten_for_broken (CORBA_Object obj,
 {
 	ORBitConnectionStatus ret;
 
-	if (ORBit_small_get_servant (obj))
+	if (!obj)
+		ret = ORBIT_CONNECTION_DISCONNECTED;
+
+	else if (ORBit_small_get_servant (obj))
 		ret = ORBIT_CONNECTION_IN_PROC;
+
 	else {
 		GIOPConnection *connection;
 
