@@ -26,9 +26,7 @@
 #include "everything.h"
 #include "constants.h"
 
-#undef TIMING_RUN
-#undef BIG_STUBS
-#define NUM_RUNS 1000
+#define NUM_RUNS 1
 
 #ifdef TIMING_RUN
 #  define d_print(a)
@@ -39,7 +37,12 @@
 extern CORBA_ORB global_orb;
 gboolean         in_proc;
 
-void
+/* Ugly- but hey */
+#define _IN_CLIENT_
+#include "server.c"
+#undef  _IN_CLIENT_
+
+static void
 testConst (void)
 {
 	d_print ("Testing constants...\n");
@@ -53,24 +56,26 @@ testConst (void)
 	g_assert (test_FAVORITE_COLOUR==test_red);
 }
 
-
-void
+static void
 testAttribute (test_TestFactory factory, 
 	       CORBA_Environment *ev)
 {  
 	test_BasicServer objref;
-	CORBA_char *val;
-	CORBA_long lval;
+	CORBA_char      *val;
+	CORBA_long       lval;
+
 	d_print ("Testing attributes...\n");
 	objref = test_TestFactory_getBasicServer (factory, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	g_assert (objref != CORBA_OBJECT_NIL);
 	g_assert (CORBA_Object_is_a (objref, "IDL:orbit/test/BasicServer:1.0", ev));
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
 	val = test_BasicServer__get_foo (objref, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);  
 	g_assert (strcmp (val, constants_STRING_RETN)==0);	
 	CORBA_free (val);
+
 	test_BasicServer__set_foo (objref, constants_STRING_IN, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
@@ -82,7 +87,7 @@ testAttribute (test_TestFactory factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testString (test_TestFactory   factory, 
 	    CORBA_Environment *ev)
 {
@@ -115,74 +120,81 @@ testString (test_TestFactory   factory,
 }
 
 
-void
+static void
 testLong (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
-  test_BasicServer objref;
-  CORBA_long inArg, inoutArg, outArg, retn;
-  d_print ("Testing longs...\n");
-  objref = test_TestFactory_getBasicServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  inArg = constants_LONG_IN;
-  inoutArg = constants_LONG_INOUT_IN;
-  retn = test_BasicServer_opLong (objref, inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  g_assert (inArg == constants_LONG_IN);
-  g_assert (inoutArg == constants_LONG_INOUT_OUT);
-  g_assert (outArg == constants_LONG_OUT);
-  g_assert (retn == constants_LONG_RETN);
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	test_BasicServer objref;
+	CORBA_long inArg, inoutArg, outArg, retn;
+
+	d_print ("Testing longs...\n");
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	inArg = constants_LONG_IN;
+	inoutArg = constants_LONG_INOUT_IN;
+	retn = test_BasicServer_opLong (objref, inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (inArg == constants_LONG_IN);
+	g_assert (inoutArg == constants_LONG_INOUT_OUT);
+	g_assert (outArg == constants_LONG_OUT);
+	g_assert (retn == constants_LONG_RETN);
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testLongLong (test_TestFactory factory, 
-		  CORBA_Environment *ev)
-{
-  test_BasicServer objref;
-  CORBA_long_long inArg, inoutArg, outArg, retn;
-  d_print ("Testing long longs...\n");
-  objref = test_TestFactory_getBasicServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  inArg = constants_LONG_LONG_IN;
-  inoutArg = constants_LONG_LONG_INOUT_IN;
-  retn = test_BasicServer_opLongLong (objref, inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  g_assert (inArg == constants_LONG_LONG_IN);
-  g_assert (inoutArg == constants_LONG_LONG_INOUT_OUT);
-  g_assert (outArg == constants_LONG_LONG_OUT);
-  g_assert (retn == constants_LONG_LONG_RETN);
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-}
-
-void testEnum (test_TestFactory factory, 
+static void
+testLongLong (test_TestFactory   factory, 
 	      CORBA_Environment *ev)
 {
-  test_BasicServer objref;
-  test_AnEnum inArg, inoutArg, outArg, retn;
-  d_print ("Testing enums...\n");
-  objref = test_TestFactory_getBasicServer (factory, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  inArg = test_ENUM_IN;
-  inoutArg = test_ENUM_INOUT_IN;
-  retn = test_BasicServer_opEnum (objref, inArg, &inoutArg, &outArg, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
-  g_assert (inArg == test_ENUM_IN);
-  g_assert (inoutArg == test_ENUM_INOUT_OUT);
-  g_assert (outArg == test_ENUM_OUT);
-  g_assert (retn == test_ENUM_RETN);
-  CORBA_Object_release (objref, ev);
-  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	test_BasicServer objref;
+	CORBA_long_long inArg, inoutArg, outArg, retn;
+
+	d_print ("Testing long longs...\n");
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	inArg = constants_LONG_LONG_IN;
+	inoutArg = constants_LONG_LONG_INOUT_IN;
+	retn = test_BasicServer_opLongLong (objref, inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (inArg == constants_LONG_LONG_IN);
+	g_assert (inoutArg == constants_LONG_LONG_INOUT_OUT);
+	g_assert (outArg == constants_LONG_LONG_OUT);
+	g_assert (retn == constants_LONG_LONG_RETN);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
+testEnum (test_TestFactory   factory, 
+	  CORBA_Environment *ev)
+{
+	test_BasicServer objref;
+	test_AnEnum inArg, inoutArg, outArg, retn;
+
+	d_print ("Testing enums...\n");
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	inArg = test_ENUM_IN;
+	inoutArg = test_ENUM_INOUT_IN;
+	retn = test_BasicServer_opEnum (objref, inArg, &inoutArg, &outArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (inArg == test_ENUM_IN);
+	g_assert (inoutArg == test_ENUM_INOUT_OUT);
+	g_assert (outArg == test_ENUM_OUT);
+	g_assert (retn == test_ENUM_RETN);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
+static void
 testException (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
 	test_BasicServer    objref;
 	test_TestException *ex;
-	CORBA_Environment  *cpyev;
+	CORBA_Environment  *cpyev, *rev = ev;
 
 	d_print ("Testing exceptions...\n");
 
@@ -216,11 +228,11 @@ testException (test_TestFactory   factory,
 
 	CORBA_free (cpyev);
 
-	CORBA_Object_release (objref, ev);  
-	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (objref, rev);
+	g_assert (rev->_major == CORBA_NO_EXCEPTION);
 }
 
-gboolean
+static gboolean
 find_tc (CORBA_sequence_CORBA_TypeCode *tcs, 
 	 const char                    *repo_id)
 {
@@ -233,7 +245,7 @@ find_tc (CORBA_sequence_CORBA_TypeCode *tcs,
 	return FALSE;
 }
 
-void
+static void
 testIInterface (test_TestFactory   factory, 
 		CORBA_Environment *ev)
 {
@@ -304,7 +316,7 @@ testIInterface (test_TestFactory   factory,
 	CORBA_free (tcs);
 }
 
-void
+static void
 testFixedLengthStruct (test_TestFactory   factory, 
 		       CORBA_Environment *ev)
 {
@@ -336,8 +348,9 @@ testFixedLengthStruct (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testVariableLengthStruct (test_TestFactory factory, 
-			      CORBA_Environment *ev)
+static void
+testVariableLengthStruct (test_TestFactory   factory, 
+			  CORBA_Environment *ev)
 {
   test_StructServer objref;
   test_VariableLengthStruct inArg, inoutArg, *outArg, *retn;
@@ -364,8 +377,9 @@ void testVariableLengthStruct (test_TestFactory factory,
 }
 
 
-void testCompoundStruct (test_TestFactory factory, 
-			CORBA_Environment *ev)
+static void
+testCompoundStruct (test_TestFactory   factory, 
+		    CORBA_Environment *ev)
 {
   test_StructServer objref;
   test_CompoundStruct inArg, inoutArg, *outArg, *retn;
@@ -392,7 +406,7 @@ void testCompoundStruct (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testStructAny (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
@@ -415,8 +429,9 @@ testStructAny (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testUnboundedSequence (test_TestFactory factory, 
-			   CORBA_Environment *ev)
+static void
+testUnboundedSequence (test_TestFactory   factory, 
+		       CORBA_Environment *ev)
 {
   test_SequenceServer objref;
   test_StrSeq inArg, inoutArg, *outArg, *retn;
@@ -468,7 +483,7 @@ void testUnboundedSequence (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testBoundedSequence (test_TestFactory   factory, 
 		     CORBA_Environment *ev)
 {
@@ -526,8 +541,9 @@ testBoundedSequence (test_TestFactory   factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testFixedLengthUnion (test_TestFactory factory, 
-			  CORBA_Environment *ev)
+static void
+testFixedLengthUnion (test_TestFactory   factory, 
+		      CORBA_Environment *ev)
 {
   test_UnionServer objref;
   test_FixedLengthUnion inArg, inoutArg, outArg, retn;
@@ -558,8 +574,9 @@ void testFixedLengthUnion (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testVariableLengthUnion (test_TestFactory factory, 
-			     CORBA_Environment *ev)
+static void
+testVariableLengthUnion (test_TestFactory   factory, 
+			 CORBA_Environment *ev)
 {
   test_UnionServer objref;
   test_VariableLengthUnion inArg, inoutArg, *outArg, *retn;
@@ -592,8 +609,9 @@ void testVariableLengthUnion (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testFixedLengthArray (test_TestFactory factory, 
-			  CORBA_Environment *ev)
+static void
+testFixedLengthArray (test_TestFactory   factory, 
+		      CORBA_Environment *ev)
 {
   
   test_ArrayServer objref;
@@ -627,8 +645,9 @@ void testFixedLengthArray (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void testVariableLengthArray (test_TestFactory factory, 
-			     CORBA_Environment *ev)
+static void
+testVariableLengthArray (test_TestFactory   factory, 
+			 CORBA_Environment *ev)
 {
   test_ArrayServer objref;
   test_StrArray inArg, inoutArg;
@@ -674,8 +693,9 @@ void testVariableLengthArray (test_TestFactory factory,
 }
 
 
-void testAnyLong (test_TestFactory factory, 
-		  CORBA_Environment *ev)
+static void
+testAnyLong (test_TestFactory   factory, 
+	     CORBA_Environment *ev)
 {
   test_AnyServer objref;
   CORBA_any inArg, inoutArg, *outArg, *retn;
@@ -727,7 +747,7 @@ void testAnyLong (test_TestFactory factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testAnyString (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
@@ -778,7 +798,7 @@ testAnyString (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testAnyStrSeq (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
@@ -798,7 +818,7 @@ testAnyStrSeq (test_TestFactory   factory,
 	CORBA_Object_release (objref, ev);
 }
 
-void
+static void
 testAnyStruct (test_TestFactory   factory, 
 	       CORBA_Environment *ev)
 {
@@ -860,7 +880,7 @@ testAnyStruct (test_TestFactory   factory,
 
 }
 
-void
+static void
 testSequenceOfAny (test_TestFactory   factory, 
 		   CORBA_Environment *ev)
 {
@@ -885,7 +905,7 @@ testSequenceOfAny (test_TestFactory   factory,
 	CORBA_free (anyseq._buffer);
 }
 
-void
+static void
 testAnyException (test_TestFactory   factory, 
 		  CORBA_Environment *ev)
 {
@@ -901,7 +921,7 @@ testAnyException (test_TestFactory   factory,
 	CORBA_free (inArg);
 }
 
-void
+static void
 testTypeCode (test_TestFactory   factory, 
 	      CORBA_Environment *ev)
 {
@@ -932,7 +952,7 @@ testTypeCode (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testContext (test_TestFactory   factory, 
 	     CORBA_Environment *ev)
 {
@@ -977,7 +997,7 @@ testContext (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
-void
+static void
 testMisc (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
@@ -993,8 +1013,8 @@ testMisc (test_TestFactory   factory,
 		g_assert (!strcmp (ev->_id, "IDL:CORBA/BAD_OPERATION:1.0"));
 		CORBA_exception_free (ev);
 	}
-
-	if (!factory->adaptor_obj) {
+	
+	if (!in_proc) {
 		test_BasicServer objref;
 
 		objref = test_TestFactory_getBasicServer (factory, ev);
@@ -1043,17 +1063,48 @@ testMisc (test_TestFactory   factory,
 	/*
 	 * dead reference check
 	 */
-	if (!factory->adaptor_obj) {
+	if (!in_proc) {
 		test_DeadReferenceObj obj;
 
 		obj = test_TestFactory_createDeadReferenceObj (factory, ev);
-
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 		g_assert (obj != CORBA_OBJECT_NIL);
 
 		test_DeadReferenceObj_test (obj, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
+		CORBA_Object_release (obj, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	} else { /* Lets do some things on a de-activated ref */
+		test_BasicServer         obj;
+		PortableServer_ObjectId *oid;
+		POA_test_BasicServer     servant = {
+			NULL, &BasicServer_vepv
+		};
+
+		obj = create_object (
+			global_poa, POA_test_BasicServer__init,
+			&servant, ev);
+
+		oid = PortableServer_POA_servant_to_id (
+			global_poa, &servant, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		PortableServer_POA_deactivate_object (
+			global_poa, oid, ev);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+		CORBA_free (oid);
+
+		test_BasicServer_opException (obj, ev);
+		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+		g_assert (!strcmp (ev->_id, ex_CORBA_OBJECT_NOT_EXIST));
+		CORBA_exception_free (ev);
+
+		test_BasicServer_opOneWay (obj, "Foo", ev);
+		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
+		g_assert (!strcmp (ev->_id, ex_CORBA_OBJECT_NOT_EXIST));
+		CORBA_exception_free (ev);
+
 		CORBA_Object_release (obj, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	}
@@ -1146,6 +1197,10 @@ test_BasicServer_opStringA_cb (CORBA_Object          object,
 	g_assert (!strcmp (out_str, constants_STRING_OUT));
 	g_assert (!strcmp (ret_str, constants_STRING_RETN));
 
+	CORBA_free (inout_str);
+	CORBA_free (out_str);
+	CORBA_free (ret_str);
+
 	done = 1;
 }
 
@@ -1168,7 +1223,7 @@ test_BasicServer_opStringA (CORBA_Object       obj,
 }
 
 
-void
+static void
 testAsync (test_TestFactory   factory, 
 	   CORBA_Environment *ev)
 {
@@ -1206,6 +1261,61 @@ testAsync (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+static void
+testPingPong (test_TestFactory   factory, 
+	      CORBA_Environment *ev)
+{
+	test_PingPongServer r_objref, l_objref, objref;
+
+	d_print ("Testing ping pong invocations ...\n");
+	r_objref = test_TestFactory_createPingPongServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	l_objref = TestFactory_createPingPongServer (NULL, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (l_objref, ev); /* only want 1 ref */
+	g_assert (ORBit_small_get_servant (l_objref) != NULL);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	test_PingPongServer_pingPong (r_objref, l_objref, 64, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	d_print ("Testing ping pong reg / lookup ...\n");
+	test_PingPongServer_set (r_objref, l_objref, "Foo", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	objref = test_PingPongServer_get (r_objref, "Foo", ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	g_assert (ORBit_small_get_servant (objref) != NULL);
+	g_assert (l_objref == objref);
+
+	d_print ("Testing equivalence\n");
+	g_assert (CORBA_Object_is_equivalent (
+		l_objref, objref, ev));
+	g_assert (CORBA_Object_is_equivalent (
+		CORBA_OBJECT_NIL, CORBA_OBJECT_NIL, ev));
+	g_assert (CORBA_Object_is_equivalent (
+		l_objref, l_objref, ev));
+	g_assert (CORBA_Object_is_equivalent (
+		r_objref, r_objref, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		r_objref, l_objref, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		l_objref, r_objref, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		l_objref, CORBA_OBJECT_NIL, ev));
+	g_assert (!CORBA_Object_is_equivalent (
+		CORBA_OBJECT_NIL, l_objref, ev));
+
+	CORBA_Object_release (objref, ev);
+
+	CORBA_Object_release (l_objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_Object_release (r_objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
 static void
 broken_cb (LINCConnection *connection, gboolean *broken)
 {
@@ -1219,7 +1329,7 @@ dummy_cb (LINCConnection *connection, gboolean *invoked)
 	*invoked = TRUE;
 }
 
-void
+static void
 testSegv (test_TestFactory   factory, 
 	  CORBA_Environment *ev)
 {
@@ -1227,6 +1337,7 @@ testSegv (test_TestFactory   factory,
 	if (!in_proc) {
 		gboolean broken = FALSE;
 		gboolean invoked = FALSE;
+		CORBA_char *id;
 
 		g_assert (ORBit_small_listen_for_broken (
 			factory, G_CALLBACK (broken_cb), &broken) ==
@@ -1240,7 +1351,8 @@ testSegv (test_TestFactory   factory,
 			factory, G_CALLBACK (dummy_cb)) ==
 			  ORBIT_CONNECTION_CONNECTED);
 
-		test_TestFactory_segv (factory, "do it!", ev); 
+		id = test_TestFactory_segv (factory, "do it!", ev); 
+#ifdef DO_HARDER_SEGV
 		g_assert (ev->_major == CORBA_SYSTEM_EXCEPTION);
 		g_assert (!strcmp (ev->_id, "IDL:CORBA/COMM_FAILURE:1.0"));
 		CORBA_exception_free (ev);
@@ -1249,10 +1361,17 @@ testSegv (test_TestFactory   factory,
 			  ORBIT_CONNECTION_DISCONNECTED);
 		g_assert (broken);
 		g_assert (!invoked);
+#else
+		g_assert (ORBit_small_unlisten_for_broken (
+			factory, G_CALLBACK (broken_cb)) ==
+			  ORBIT_CONNECTION_CONNECTED);
+		g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		CORBA_free (id);
+#endif
 	}
 }
 
-void
+static void
 test_initial_references (CORBA_ORB          orb,
 			 CORBA_Environment *ev)
 {
@@ -1295,20 +1414,14 @@ run_tests (test_TestFactory   factory,
 {
 	int i;
 
-#ifdef TIMING_RUN
 	for (i = 0; i < NUM_RUNS; i++) {
-#else
-	for (i = 0; i < 1; i++) {
-#endif
 		testConst ();
 		testAttribute (factory, ev);
 		testString (factory, ev);
 		testLong (factory, ev);
 		testLongLong (factory, ev);
 		testEnum (factory, ev);
-#ifndef BIG_STUBS
 		testException (factory, ev);
-#endif
 		testFixedLengthStruct (factory, ev);
 		testVariableLengthStruct (factory, ev);
 		testCompoundStruct (factory, ev);
@@ -1317,7 +1430,6 @@ run_tests (test_TestFactory   factory,
 		testBoundedSequence (factory, ev);
 		testFixedLengthUnion (factory, ev);
 		testVariableLengthUnion (factory, ev);
-#ifndef BIG_STUBS
 		testFixedLengthArray (factory, ev);
 		testVariableLengthArray (factory, ev);
 		testAnyStrSeq (factory, ev);
@@ -1331,23 +1443,18 @@ run_tests (test_TestFactory   factory,
 		testIInterface (factory, ev);
 		testMisc (factory, ev);
 		testAsync (factory, ev);
-#endif
-
-#ifndef TIMING_RUN
+		if (!in_proc)
+			testPingPong (factory, ev);
+#if NUM_RUNS == 1
 		testSegv (factory, ev);
 #endif
 	}
 	
-#ifdef TIMING_RUN
+#if NUM_RUNS > 1
 	g_warning ("Did '%d' iterations", i);
 #endif
 }
 
-/* Ugly- but hey */
-#define _IN_CLIENT_
-#include "server.c"
-#undef  _IN_CLIENT_
-
 int
 main (int argc, char *argv [])
 {
@@ -1369,20 +1476,16 @@ main (int argc, char *argv [])
 	/* In Proc ... */
 	in_proc = TRUE;
 
-#ifndef TIMING_RUN
 	fprintf (stderr, "\n --- In proc ---\n\n\n");
-	{
-		factory = get_server (global_orb, &ev);
-		g_assert (factory->profile_list == NULL);
-		g_assert (ORBit_object_get_connection (factory) == NULL);
-		run_tests (factory, &ev);
-		CORBA_Object_release (factory, &ev);
-		g_assert (ev._major == CORBA_NO_EXCEPTION);
-		factory = CORBA_OBJECT_NIL;
-	}
-	fprintf (stderr, "\n\n --- Out of proc ---\n\n\n");
-#endif
+	factory = get_server (global_orb, &ev);
+	g_assert (factory->profile_list == NULL);
+	g_assert (ORBit_object_get_connection (factory) == NULL);
+	run_tests (factory, &ev);
+	CORBA_Object_release (factory, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
+	factory = CORBA_OBJECT_NIL;
 
+	fprintf (stderr, "\n\n --- Out of proc ---\n\n\n");
 	in_proc = FALSE;
 
 	{ /* read the ior from iorfile, and swizzle to an objref*/
@@ -1409,6 +1512,11 @@ main (int argc, char *argv [])
 
 	CORBA_Object_release (factory, &ev);
 	g_assert (ev._major == CORBA_NO_EXCEPTION);
+
+	g_warning ("released factory");
+
+	CORBA_Object_release ((CORBA_Object) global_poa, &ev);
+	g_assert (ev._major == CORBA_NO_EXCEPTION);
 	
 	CORBA_ORB_destroy (global_orb, &ev);
 	CORBA_exception_free (&ev);
